	;; temperature sensor functions
	;; derived from ds1820.asm, gpl

	;; ??? todo, fix for ds18s20 chips
	
p_ts		equ	portb		; keyboard column outputs
t_ts		equ	trisb
b_sensor	equ	3
m_sensor	equ	(1<<b_sensor)

	cblock
		r_trism		; mirror of tristate output latch
				; used by ow_high, ow_low, main
	
		r_which		; bit mask of which DS1820 to talk to
	
				; data read from device scratchpad
		r_ds_lsb	; temperature lsb
		r_ds_msb	; temperature msb
		r_ds_rem	; remaining count after conversion
		r_ds_count	; counts per degrees c
		r_ds_crc	; cyclic redundancy check byte
	endc

ts_get
	;; select sensor
	movlw	m_sensor
	movwf	r_which

	;; grab the tristate latch
	bsf	status,rp0
	movf	p_ts,w
	bcf	status,rp0
	movwf	r_trism

	;; reset bus to determine if sensor is present
	call	ow_reset		; ( -- flag )
	popw
	iorlw	0x0			; set zero flag if sensor absent
	bnz	ts_get_present

ts_get_absent
	pushl	'?'
	goto	lcd_put
	
ts_get_present

	;; skip rom check sequence
	pushl	ow_command_skip
	call	ow_tx

	;; commence conversion
	pushl	ow_command_convert
	call	ow_tx

	;; wait until sensor not busy
	
	;; ??? we wait until we get eight bits of 1's, which could take
	;; longer than just waiting for one bit of 1.  try waiting for 1?

	;; ds1820 specification says 200ms typical, 500ms maximum
ts_busy
	btfsc	flag_kb			; if keyboard detected
	return				; give up
	btfsc	flag_1hz		; if end of second detected
	return				; give up
	
	call	ow_rx			; fetch a byte
	popw				; pop the byte from stack
	xorlw	0xff			; test for 0xff
	bnz	ts_busy			; loop if not equal

	;; ask sensor for data

	;; reset bus
	call	ow_reset		; ( -- flag )
	popw
	iorlw	0x0			; set zero flag if sensor absent
	skpnz
	goto	ts_get_absent

	;; skip rom check sequence
	pushl	ow_command_skip
	call	ow_tx

	;; request data
	pushl	ow_command_read
	call	ow_tx
	
	;; read data from sensor
ow_get	macro	m_byte		; get byte to address
	call	ow_rx
	popf	m_byte
	endm
	
	ow_get	r_ds_lsb	; temperature, least significant bits
	ow_get	r_ds_msb	; temperature, most significant bits
	ow_get	r_ds_rem	; ignore low alarm byte
	ow_get	r_ds_rem	; ignore high alarm byte
	ow_get	r_ds_rem	; ignore reserved byte
	ow_get	r_ds_rem	; ignore reserved byte
	ow_get	r_ds_rem	; temperature, remainder
	ow_get	r_ds_count	; temperature, correction count
	ow_get	r_ds_crc	; cyclic redundancy check

	;; ??? check the crc some day!

;;;
;;; Calculation of high resolution temperature from the count per degrees
;;; c and remaining count values fetched from the DS1820 scratchpad.
;;;
;;; high-res-temp = ( temperature >> 1 ) - 0.25
;;;                 + (count-per-c - count-remain) / (count-per-c)
;;; 
;;; Problem:	Result is truncated to two decimal places rather than being
;;;		rounded.  Solution not yet planned.
;;; 

	;; move temperature as read to accumulator
	pushf16	r_ds_lsb

	;; shift right to truncate 0.5C LSB
	bcf	status,c	; start with a clear carry flag
	rrf	indf,f
	incf	fsr,f
	rrf	indf,f
	decf	fsr,f

	;; multiply by 100 to scale to hundredths of degrees
	pushl16	d'100'
	call	sb_multiply

	;; subtract 25 from accumulator
	pushl16	d'25'
	call	sb_subtract

	;; keep on stack for later use

	;; take eight bit count per degrees c
	;; subtract count remaining
	movf	r_ds_rem,w	; get count remaining
	subwf	r_ds_count,w	; subtract it from count per degrees c
	pushw			; place on stack
	pushl	d'0'		; extend to sixteen bits

	;; multiply by 100 to scale
	pushl16	d'100'
	call	sb_multiply

	;; divide by count per degrees c
	pushf	r_ds_count
	pushl	d'0'			; extend to sixteen bits
	call	sb_divide
	call	sb_nip			; drop remainder

	;; we now have offset from temperature in hundredths of degrees
	;; add to previouosly saved value

	call	sb_add

	;; we now have the measurement in degrees centigrade, scaled by
	;; 100, such that 23.54 degrees is binary value for decimal 2354
	
	;; check sign of result
	btfss	indf,7		; sign bit of signed integer result
	goto	skip		; bit cleared, it is positive

	;; issue negative sign to output
	pushl	'-'
	call	lcd_put

	;; subtract from zero
	call	sb_negate

skip
	
	;; convert value to bcd
	call	sb_bcd

	;; transmit the bcd output buffer
	drop				; d2 (msd)
	call	lcd_put_hex		; d1
	pushl	'.'			; interject the decimal place '0655.35'
	call	lcd_put
	goto	lcd_put_hex		; d0 (lsd)
